#!/usr/bin/perl
#
# MERLIN_SCAN_START.PL
# Start script for genome wide Merlin runs with sets of SNPs
# (c) 2006 Franz Rueschendorf 

use Cwd;
use strict;
use File::Copy;
use File::Basename;

my @files=();
my $prog="merlin";
my $command="";

# foreach my $chr ("16") {	# edit and use this line to calculate one chromosome and comment out the next line

foreach my $chr ("01","02","03","04","05","06","07","08","09","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24") {
	if ( -d "c$chr") {
		
		chdir "c${chr}";
		print "Chromosome $chr\n";

		my $set=0;
		@files=glob("datain_*.${chr}");
		$set=@files;

#----------------- modify genetic model here  -------------#
		open (OUT,"> param.tbl") || die "could not open param.tbl :$!\n";
#		print OUT "TRAIT   0.0010 0.00,1.00,1.00 Rare_Dominant\n";
#		print OUT "TRAIT   0.0001 0.00,0.00,1.00 Rare_Recessive\n";

		if ($chr == 23) {print OUT "TRAIT   0.001 0.00,0.99,0.99 Rare_Dominant\n";}
		else {print OUT "TRAIT   0.001 0.00,1.00,1.00 Rare_Dominant\n";}

		close (OUT);
#--------------------- end genetic model ------------------#

		if (-f "merlin.chr") {unlink "merlin.chr";}
		if (-f "merlin.out") {unlink "merlin.out";}
		unlink glob("merlin_*.chr");
		unlink glob("merlin_*.out");
		unlink glob("wiped_*.*");
			
		if ($chr == 23) {$prog = "minx";} else {$prog = "merlin";}
		
		foreach my $i ( 1..$set) {	

#------------- modify merlin commands here ----------------#

			### non-parametric, parametric and error detection
			$command = "$prog -d datain_$i.$chr -p pedin_$i.$chr -m merlin_map_$i.$chr --quiet --error --npl --model > merlin_$i.out";

			### non-parametric and parametric and haplotyping
#			$command = "$prog -d datain_$i.$chr -p pedin_$i.$chr -m merlin_map_$i.$chr --quiet --error --best --npl --model > merlin_$i.out";

			system ("$command");
			if ( -f "merlin.chr") { move ("merlin.chr","merlin_$i.chr");}
			if ( -f "merlin.flow") { move ("merlin.flow","merlin_$i.flow");}

			### modeling LD
#			$command = "$prog -d datain_$i.$chr -p pedin_$i.$chr -m merlin_map_$i.$chr --npl  --grid 2 --rsq 0.2 --cfreq > rsq0.2_merlin_$i.out";
#			system ("$command");

			### wipe unlikely genotypes with pedwipe and run Merlin again
#			$command = "pedwipe -d datain_$i.$chr -p pedin_$i.$chr";
#			system ("$command");
#			$command = "$prog -d wiped.dat -p wiped.ped -m wiped.map -f wiped.freq --megabytes 1200 --best --quiet --npl --model > wiped_$i.out";
#			system ("$command");
		}
		chdir  ".." ;
	}
	else {
		print "no chromosome $chr\n";
	}
}
